//////////////////↓環境に合わせて変更するパラメーター///////////////////////////////////
// 管理者より払い出されるAPIトークンを設定してください。
// 払い出し方については下記の活用支援サイトをご確認ください。
// https://www.e-sales-success.jp/guide/remix/web/knowledge1225.html

const X_Auth_API_Token = "xxxx";

//////////////////↑環境に合わせて変更するパラメーター///////////////////////////////////

const schCodeStr = "sch_code";
const schIconType = "sch_type";
const startDateStr = "start_date";
const schTypeStr = "meeting_type";
const normalMeetingBtnId = "attendAllSameNameNormalMeetingBtn";
const regularMeetingBtnId = "attendAllRegularMeetingBtn";
const allMeetingBtnId = "attendAllMeetingBtn";

async function getAttendingRegularMeetingDataByApi() {
  const sch_code_sch_type_start_date_InfoArray = new Array();
  const body = createSearchRequestBody(
    "schedule",
    createRegularMeetingsSearchCondition,
    createColumnCodesListForRequest(
      getscheduleColumnCode(schCodeStr),
      getscheduleColumnCode(schIconType),
      getscheduleColumnCode(startDateStr)
    ),
    createSortKeys(getscheduleColumnCode(schCodeStr), false)
  );

  const apiInfo = createApiInfo("search");
  const url = apiInfo.path;
  await fetch(url, {
    method: "post",
    headers: {
      "Content-Type": "application/json",
      "X-Auth-API-Token": X_Auth_API_Token,
    },
    body: JSON.stringify(body),
  })
    .then((res) => {
      return res.json();
    })
    .then((data) => {
      let resValues = data.values;
      let itemData;
      for (let i = 0; i < resValues.length; i++) {
        let obj = new Object();
        itemData = resValues[i].item_data;
        obj[schCodeStr] = itemData[0].num;
        obj[schIconType] = itemData[1].num;
        obj[startDateStr] = itemData[2].date
          .substring(0, 10)
          .replaceAll("-", "");
        sch_code_sch_type_start_date_InfoArray.push(obj);
      }
    });
  return sch_code_sch_type_start_date_InfoArray;
}

async function getScheduleEntity(sch_code) {
  let apiInfo = createApiInfo("entity", "GET");
  const url =
    apiInfo.path +
    `?obj=schedule&id=${sch_code}&columncodes=${getscheduleColumnCode(
      "meeting_type"
    )},${getscheduleColumnCode("subject")},1406`;
  return await fetch(url, {
    method: apiInfo.method,
    headers: {
      "X-Auth-API-Token": X_Auth_API_Token,
    },
  })
    .then((res) => {
      return res.json();
    })
    .then((data) => {
      const itemData = data.value.item_data;
      return {
        meeting_type: itemData[0].num,
        subject: itemData[1].text,
        regCode: itemData[2].num,
      };
    });
}

async function getEmpEntity(emp_code) {
  let apiInfo = createApiInfo("entity", "GET");
  const url =
    apiInfo.path +
    `?obj=employee&id=${emp_code}&columncodes=${getscheduleColumnCode("502")}`;
  return await fetch(url, {
    method: apiInfo.method,
    headers: {
      "X-Auth-API-Token": X_Auth_API_Token,
    },
  }).then((res) => {
    return res.json();
  });
}

async function getAttendingNormalMeetingDataByApi() {
  const sch_code_sch_type_start_date_InfoArray = new Array();
  const body = createSearchRequestBody(
    "schedule",
    createNormalMeetingsSearchCondition,
    createColumnCodesListForRequest(
      getscheduleColumnCode(schCodeStr),
      getscheduleColumnCode(schIconType),
      getscheduleColumnCode(startDateStr)
    ),
    createSortKeys(getscheduleColumnCode(schCodeStr), false)
  );
  const apiInfo = createApiInfo("search");
  const url = apiInfo.path;
  await fetch(url, {
    method: "post",
    headers: {
      "Content-Type": "application/json",
      "X-Auth-API-Token": X_Auth_API_Token,
    },
    body: JSON.stringify(body),
  })
    .then((res) => {
      return res.json();
    })
    .then((data) => {
      let resValues = data.values;
      let itemData;
      for (let i = 0; i < resValues.length; i++) {
        let obj = new Object();
        itemData = resValues[i].item_data;
        obj[schCodeStr] = itemData[0].num;
        obj[schIconType] = itemData[1].num;
        obj[startDateStr] = itemData[2].date
          .substring(0, 10)
          .replaceAll("-", "");
        sch_code_sch_type_start_date_InfoArray.push(obj);
      }
    });
  return sch_code_sch_type_start_date_InfoArray;
}

function getTodayDate() {
  var dt = new Date();
  var y = dt.getFullYear();
  var m = ("00" + (dt.getMonth() + 1)).slice(-2);
  var d = ("00" + dt.getDate()).slice(-2);
  return y + "-" + m + "-" + d;
}

function createNormalMeetingsSearchCondition() {
  let myEmpCodeConditionItem = {
    columnCode: getscheduleColumnCode("employee_code"),
    operator: "=",
    num: getMyEmpCodeFromScheduleCalender(),
  };
  let meetingSubjectConditionItem = {
    columnCode: getscheduleColumnCode("subject"),
    operator: "=",
    text: getSubjectFromScheduleTip(),
  };
  let attendanceStateItem = {
    columnCode: getscheduleColumnCode("reply_code"),
    operator: "=",
    num: getAttendanceStateValue("notChecked"),
  };
  let startDayIsAfterTodayItem = {
    columnCode: getscheduleColumnCode("start_date"),
    operator: ">=",
    date: getTodayDate(),
  };
  const NormalMeetingsAddedAsAttendee_SearchConditionItems = [
    myEmpCodeConditionItem,
    meetingSubjectConditionItem,
    attendanceStateItem,
    startDayIsAfterTodayItem,
  ];

  return NormalMeetingsAddedAsAttendee_SearchConditionItems;
}

function createRegularMeetingsSearchCondition() {
  let myEmpCodeConditionItem = {
    columnCode: getscheduleColumnCode("employee_code"),
    operator: "=",
    num: getMyEmpCodeFromScheduleCalender(),
  };
  let regularMeetingCodeConditionItem = {
    columnCode: getscheduleColumnCode("regular_code"),
    operator: "=",
    num: getRegularCodeFromscheduleDetail(),
  };
  let attendanceStateItem = {
    columnCode: getscheduleColumnCode("reply_code"),
    operator: "=",
    num: getAttendanceStateValue("notChecked"),
  };
  let startDayIsAfterTodayItem = {
    columnCode: getscheduleColumnCode("start_date"),
    operator: ">=",
    date: getTodayDate(),
  };
  const RegularMeetingsAddedAsAttendee_SearchConditionItems = [
    myEmpCodeConditionItem,
    regularMeetingCodeConditionItem,
    attendanceStateItem,
    startDayIsAfterTodayItem,
  ];

  return RegularMeetingsAddedAsAttendee_SearchConditionItems;
}

function getAttendanceStateValue(value) {
  const notChecked = 0;
  const attend = 1;
  const absence = 2;
  let state;
  switch (value) {
    case "notChecked":
      state = notChecked;
      break;
    case "attend":
      state = attend;
      break;
    case "absence":
      state = absence;
      break;
    default:
      console.error("invalid command");
  }
  return state;
}
function getMeetingTypeValue(typeName) {
  let meetingType = null;
  const normal = 1;
  const wholeDay = 2;
  const longTime = 3;
  const regular = 4;
  switch (typeName) {
    case "regular":
      meetingType = regular;
      break;
    case "normal":
      meetingType = normal;
      break;
    case "wholeDay":
      meetingType = wholeDay;
      break;
    case "longTime":
      meetingType = longTime;
      break;
    default:
      meetingType = null;
  }
  return meetingType;
}

function getRegularCodeFromscheduleDetail() {
  let val = schEntity.regCode;
  return val;
}

let regSchCode = null;
function getRegularCodeFromEntity() {
  regSchCode = schEntity;
  let val = document.deleteAll.regular_code.value;
  return val;
}

function getMyEmpCodeFromScheduleCalender() {
  const regexForEmpCode = /employee_code=\d+/;
  const regexForNum = /\d+/;
  let tag =
    iframeDoc.getElementById("pageTool").firstElementChild.firstElementChild
      .firstElementChild;
  let empCode = tag
    .getAttribute("onclick")
    .match(regexForEmpCode)[0]
    .match(regexForNum)[0];

  return empCode;
}

function getMeetingIconTypeFromScheduleDetail() {
  let val = document.myform.icon_type.value;
  return val;
}

function getSchCodeFromscheduleDetail() {
  let val = document.myform.sch_code.value;
  return val;
}

function getSubjectFromScheduleTip() {
  let val = iframeDoc.getElementById("qtip-schToDoCommon-content")
    .firstElementChild.firstElementChild.nextElementSibling.nextElementSibling
    .firstElementChild.textContent;
  return val;
}

function createSearchCondition(objName, createSearchConditionFunc) {
  const searchCondition = {
    targetObjectName: objName,
    items: createSearchConditionFunc(),
    notExists: true,
    relatedObjectConditions: [],
  };
  return searchCondition;
}

function getscheduleColumnCode(codeName) {
  let code;
  switch (codeName) {
    case "sch_code":
      code = 1301;
      break;
    case "employee_code":
      code = 1302;
      break;
    case "start_date":
      code = 1303;
      break;
    case "end_date":
      code = 1304;
      break;
    case "subject":
      code = 1305;
      break;
    case "sch_type":
      code = 1403;
      break;
    case "regular_code":
      code = 1406;
      break;
    case "reply_code":
      code = 1413;
      break;
    case "team_code":
      code = 1416;
      break;
    case "meeting_type":
      code = 1417;
      break;
    default:
      console.error("invalid command");
  }

  return code;
}

function createSortKeys(columnCode, descState) {
  let sortKeysArray = [
    {
      columnCode: columnCode,
      desc: descState,
    },
  ];
  return sortKeysArray;
}

function createSearchRequestBody(
  objName,
  createSearchConditionFunc,
  columnCodeArray,
  sortKeysArray
) {
  let body = {
    searchCondition: createSearchCondition(objName, createSearchConditionFunc),
    columnCodes: columnCodeArray,
    sortKeys: sortKeysArray,
    fromIndex: 1,
  };

  return body;
}

function createColumnCodesListForRequest(...args) {
  const columnCodesListArray = new Array();
  args.forEach(function (item, index) {
    columnCodesListArray.push(item);
  });

  return columnCodesListArray;
}

class ApiInterface {
  constructor(obj) {
    this.obj = obj;
    this.Apipath = obj.path;
    this.path = obj.method;
  }
}

function createApiInfo(command, method) {
  const ApiURLList = {
    names: "entities/names",
    checkitems: "entities/checkitems",
    selectitems: "entities/selectitems",
    columns: "entities/columns",
    search: "entities/search",
    entity: "entity",
    entities: "entities",
  };

  let saveCommand;
  let requestMethod = "GET";
  if (method == void 0) {
    if (command == "search") {
      requestMethod = "POST";
    }
  } else requestMethod = method;

  switch (command) {
    case "names":
      saveCommand = ApiURLList.names;
      break;
    case "checkitems":
      saveCommand = ApiURLList.checkitems;
      break;
    case "selectitems":
      saveCommand = ApiURLList.selectitems;
      break;
    case "columns":
      saveCommand = ApiURLList.columns;
      break;
    case "search":
      saveCommand = ApiURLList.search;
      break;
    case "entity":
      saveCommand = ApiURLList.entity;
      break;
    case "entities":
      saveCommand = ApiURLList.entities;
      break;
    default:
      console.error("invalid command");
  }

  const path = getApiBasePath() + saveCommand;
  const ApiInfo = {
    path: path,
    method: requestMethod,
  };
  return ApiInfo;
}

function getPathByContext() {
  const contextPath = location.pathname.match(/\/.*\//);
  const url = location.protocol + "//" + location.host + contextPath;

  return url;
}

function getApiBasePath() {
  const eSMRestPath = "rest/v1/";
  const url = getPathByContext();
  const path = url + eSMRestPath;

  return path;
}

async function sleep(ms) {
  await new Promise((s) => setTimeout(s, ms));
}

async function attendAllMeeting(meetingCodesListArray) {
  const emp_code = getMyEmpCodeFromScheduleCalender();
  let attendance_status = getAttendanceStateValue("attend");
  let sch_code = null;
  let start_date = null;

  showLoadPage();
  const PathByContext = getPathByContext();

  for (let i = 0; i < meetingCodesListArray.length; i++) {
    sch_code = meetingCodesListArray[i][schCodeStr];
    start_date = meetingCodesListArray[i][startDateStr];
    if (i + (1 % 11) == 0) {
      await sleep(1000);
    }
    await fetch(
      `${PathByContext}esales-pc?page=plan_modify&employee_code=${emp_code}&sch_code=${sch_code}&user_sch_code=${sch_code}&command=reply&icon_type=${attendance_status}&from_chip=true&iTop=0&iLeft=0&date=${start_date}&mode=mo&multiple=false&modal=`,
      {
        headers: {
          accept:
            "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
          "accept-language": "ja,en-US;q=0.9,en;q=0.8",
          "sec-ch-ua":
            '" Not A;Brand";v="99", "Chromium";v="102", "Google Chrome";v="102"',
          "sec-ch-ua-mobile": "?0",
          "sec-ch-ua-platform": '"Windows"',
          "sec-fetch-dest": "iframe",
          "sec-fetch-mode": "navigate",
          "sec-fetch-site": "same-origin",
          "sec-fetch-user": "?1",
          "upgrade-insecure-requests": "1",
        },
        method: "GET",
      }
    )
      .then((res) => {
        if (!res.ok) {
          throw new Error("network connection was not OK");
        }
      })
      .catch((err) => {
        console.error(err);
        alert("通信に失敗しました。もう一度試してください");
        reloadPages();
      });
  }
}

async function attendAllRegularMeetingBtn(schCodesArray) {
  await attendAllMeeting(schCodesArray);
}

async function setAttendAllMeetingBtn(meetingType) {
  let form_btns = document.top_form;
  let btn = form_btns.firstElementChild.firstElementChild;
  let newlist = document.createElement("li");
  let id = null;
  if (meetingType == getMeetingTypeValue("normal")) {
    newlist.innerHTML = `<a href='javascript:void(0)' id='${allMeetingBtnId}'>同名全出席</a>`;
    id = allMeetingBtnId;
  } else if (meetingType == getMeetingTypeValue("regular")) {
    newlist.innerHTML = `<a href='javascript:void(0)' id='${allMeetingBtnId}'>定例全出席</a>`;
    id = allMeetingBtnId;
  } else {
    return id;
  }
  form_btns.firstElementChild.insertBefore(newlist, btn);
  newlist.classList.add(btn.className);
  return id;
}

function replyFormVanisher() {
  let formParent = document.getElementById("schedule_items");
  formParent.querySelector("#reply_area_base").outerHTML = "";
}

function reloadParentTab() {
  const parentTabNotExists = null;
  const parentTab = window.opener;
  if (parentTab != parentTabNotExists) {
    parentTab.location.reload();
  }
}

function reloadCurrentPage() {
  window.location.reload();
}

function reloadPages() {
  reloadParentTab();
  reloadCurrentPage();
}

function showLoadPage() {
  const loading_background = "loading_background";
  const loading_circle_container = "loading_circle_container";
  const loading_circle_array = [
    "loading_circle1",
    "loading_circle2",
    "loading_circle3",
  ];
  const circle_base = "circle_base";
  const loadmsg = "loadmsg";

  const loadPageContainer = document.createElement("div");
  loadPageContainer.setAttribute("id", "load_page_container");

  const laodPage = document.createElement("div");
  laodPage.classList.add(loading_background);

  const loading_circle_container_div = document.createElement("div");
  loading_circle_container_div.classList.add(loading_circle_container);

  for (let i = 0; i < loading_circle_array.length; i++) {
    let span_tag = document.createElement("span");
    span_tag.classList.add(circle_base);
    span_tag.classList.add(loading_circle_array[i]);
    loading_circle_container_div.appendChild(span_tag);
  }

  let loadMsg = document.createElement("span");
  loadMsg.classList.add(loadmsg);

  const boldMsg = document.createElement("b");
  boldMsg.setAttribute("id", "bold_load_msg");
  boldMsg.textContent = "処理中です。画面を閉じずにお待ちください";

  loadMsg.appendChild(boldMsg);
  loading_circle_container_div.appendChild(loadMsg);
  loadPageContainer.appendChild(laodPage);
  loadPageContainer.appendChild(loading_circle_container_div);
  document.body.appendChild(loadPageContainer);
}

function loadPageVanisher() {
  document.getElementById("load_page_container").outerHTML = "";
}

async function listenerCallback(schcode) {
  let link_css = document.createElement("link");
  link_css.setAttribute("rel", "stylesheet");
  link_css.setAttribute("href", "./loading.css");
  if (schduleInfo.isFetchedSchCode(schcode) == false) {
    schEntity = await getScheduleEntity(schcode);
    let dataByApiFunc = null;
    if (schEntity.meeting_type == getMeetingTypeValue("normal")) {
      dataByApiFunc = getAttendingNormalMeetingDataByApi;
    } else if (schEntity.meeting_type == getMeetingTypeValue("regular")) {
      dataByApiFunc = getAttendingRegularMeetingDataByApi;
    }
    schduleInfo.setSchCodesListArray(schcode, await dataByApiFunc());
    for (let i = 0; i < schduleInfo[schcode].length; i++) {
      schduleInfo[schduleInfo[schcode][i][schCodeStr]] = schduleInfo[schcode];
    }
  }
  let numOfMtgs = schduleInfo[schcode].length;
  if (numOfMtgs <= 1) {
    return;
  }

  let btnId = await a_setAttendAllMeetingBtn(schEntity.meeting_type);
  if (btnId == alreadyExist) {
    return;
  }
  let confirmMsg = null;
  if (btnId == null) {
    return;
  }
  let btnTag = iframeDoc.getElementById(allMeetingBtnId);
  if (btnId == normalMeetingBtnId) {
    confirmMsg = `本日以降、${numOfMtgs}件の参加状態が未確認の同名の会議が存在します。\n全ての会議に参加しますか？`;
  } else if (btnId == regularMeetingBtnId) {
    confirmMsg = `本日以降、${numOfMtgs}件の参加状態が未確認の定例会議が存在します。\n全ての会議に参加しますか？`;
  }
  btnTag.addEventListener("click", async function () {
    let result = confirm(confirmMsg);
    if (result == false) {
      return;
    }
    await attendAllMeeting(schduleInfo[schcode]);
    reloadPages();
  });
}

async function sleep(ms) {
  await new Promise((s) => setTimeout(s, ms));
}

let iframeDoc = null;

const observer = new MutationObserver((mutations) => {
  mutations.forEach(async (mutation) => {
    let ChildOfTipTag = null;
    let tipTag = document.getElementById("qtip-schToDoCommon");
    if (checkExistanceOfTag(tipTag) == false) {
      return;
    }
    for (let i = 0; i < 25; i++) {
      ChildOfTipTag = iframeDoc.getElementById("qtip-schToDoCommon-content");
      if (
        checkExistanceOfTag(ChildOfTipTag) == false &&
        checkExistanceOfTag(tipTag) == false
      ) {
        return;
      } else if (
        checkExistanceOfTag(ChildOfTipTag) == false &&
        checkExistanceOfTag(tipTag) == true
      ) {
        await sleep(64);
        continue;
      } else if (
        checkExistanceOfTag(ChildOfTipTag) == true &&
        checkExistanceOfTag(tipTag) == true
      ) {
        break;
      }
    }
    schcode = ChildOfTipTag.firstChild.getAttribute("schcode");
    listenerCallback(schcode);
  });
});

let iframeBody = null;
function setIframe() {
  let getiframeBody = document.body;
  let getiframeDoc = document;
  iframeDoc = getiframeDoc;
  iframeBody = getiframeBody;
}

window.addEventListener("load", () => {
  new Promise((resolve) => {
    init();
    resolve();
  });
});

async function init() {
  setIframe();
  turnOnObserver();
}

function turnOnObserver() {
  observer.observe(iframeBody, config);
}

let schEntity = null;
let meetingType = null;
let dataByApiFunc = null;

const config = {
  childList: true,
};

function checkExistanceOfTag(tag) {
  let returnVal = null;
  if (tag == null) {
    returnVal = false;
  } else if (tag != null) {
    returnVal = true;
  }
  return returnVal;
}

let tip = document.getElementById("qtip-schToDoCommon");
let schcode = null;

const alreadyExist = "exist";

async function a_setAttendAllMeetingBtn(meetingType) {
  if (iframeDoc.getElementById("attendAllMeetingBtn") != null) {
    return alreadyExist;
  }
  let btnType = null;
  const childSpanOfContent = iframeDoc
    .getElementById("qtip-schToDoCommon-content")
    .querySelector("span");
  const parentUlOfLi = childSpanOfContent.querySelector("ul");
  let btn = document.createElement("li");
  btn.setAttribute("style", "padding-left:8px;margin:0px;");
  let id = null;
  if (meetingType == getMeetingTypeValue("normal")) {
    btn.innerHTML = `<a href='javascript:void(0)' id='${allMeetingBtnId}'>一括出席</a>`;
    btnType = normalMeetingBtnId;
    id = allMeetingBtnId;
  } else if (meetingType == getMeetingTypeValue("regular")) {
    btn.innerHTML = `<a href='javascript:void(0)' id='${allMeetingBtnId}'>一括出席</a>`;
    btnType = regularMeetingBtnId;
    id = allMeetingBtnId;
  } else {
    return btnType;
  }
  parentUlOfLi.insertBefore(btn, parentUlOfLi.firstChild);
  return btnType;
}

const schduleInfo = {
  isFetchedSchCode: isFetchedSchCode,
  setSchCodesListArray: setSchCodesListArray,
  isLastSchCodeAlreadyFetched: false,
};

function isFetchedSchCode(schCode, undefVar) {
  let alreadyFetched = true;
  if (schduleInfo[schCode] == undefVar) {
    alreadyFetched = false;
  }
  this.aleardyFetched = alreadyFetched;
  return alreadyFetched;
}

function setSchCodesListArray(schCode, schCodeListArray) {
  schduleInfo[schCode] = schCodeListArray;
}
